/*
 * Copyright (C) 2000 Gene Kan.  All rights reserved.  genehkan@xcf.berkeley.edu
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    or portions of this software must clearly produce the following
 *    acknowledgement, visibly or audibly:
 *
 *      This product includes portions of Litestream MP3 Streaming System.
 *
 * 4. All products derived from this software or portions of this software
 *    must clearly produce the following acknowledgement to each user of
 *    that product, visibly or audibly:
 *
 *      This product includes portions of Litestream MP3 Streaming System.
 *
 *    For example: Each listener connecting to an MP3 streaming server
 *    derived from this software or portions of this software must be
 *    clearly notified that the MP3 streaming server to which he is
 *    connecting "includes portions of Litestream MP3 Streaming System."
 *
 * 5. The intention of items 3 and 4 above is to force products derived
 *    from this software or portions of this software to give credit to
 *    this software.
 *
 * 6. The name Gene Kan may not be used to endorse products derived from
 *    this software or portions of this software without specific prior
 *    written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY GENE KAN ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 * NO EVENT SHALL GENE KAN BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE OR PORTIONS OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY
 * OF SUCH DAMAGE.
 *
 */

#include <stdio.h>
#include <string.h>
#include <errno.h>

#include "stream_std.h"
#include "playlist.h"

#include "textutils.h"

FILE* open_next_file(char* _files[], int _nfiles, int* _curfile) {
	int startfile = *_curfile;
	FILE* fp;
	char message[1024];

	do {
		fp = fopen(_files[*_curfile], "r");

		if (!fp) {
			sprintf(message, "could not open %s", _files[*_curfile]);
			stream_err(message);
		}
		else {
			sprintf(message, "opened %s", _files[*_curfile]);
			stream_msg(message);
		}

		*_curfile = (*_curfile + 1) % _nfiles;

		if (fp)
			break;

		if (*_curfile == startfile) {
			stream_msg("no reasonable files found");
			return NULL;
		}
	} while (!fp);

	return fp;
}

playlist* read_playlist(const char* _filename) {
	FILE* fp;

	playlist* list;

	char filename[1024];

	list = (playlist*) malloc(sizeof(playlist));
	list->used = 0;
	list->size = 16;

	list->names = (char**) malloc(sizeof(char*) * list->size);

	if (!(fp = fopen(_filename, "r")))
		return NULL;

	while (!feof(fp) && !ferror(fp)) {
		if (!(fgets(filename, sizeof(filename), fp)))
			break;

		if (list->used == list->size) {
			list->size *= 2;
			list->names = (char**) realloc(list->names, sizeof(char*) * list->size);
		}

		list->names[list->used++] = strdup(stripspaces(filename, LEADING | TRAILING));
	}

	fclose(fp);

	return list;
}
